﻿--
-- The pricelist_det table allows the same product / product_type to be on the table more than once
-- We need to remove the duplicates and create a constraint
create temp table dc_pricelistfix (pricelist integer, product integer, product_type integer, sell_price numeric(8,2));
insert into dc_pricelistfix select distinct pricelist, product, product_type, sell_price from pricelist_det;
delete from pricelist_det;
insert into pricelist_det (pricelist, product, product_type, sell_price) select * from dc_pricelistfix;
drop table dc_pricelistfix;
create unique index pricelistdet_uq on pricelist_det(pricelist, product, product_type);
--
-- Create new tables for hire discount.  disc_det was relationally mapped to two entities
--
create table hire_disc_detail (
nsuk serial NOT NULL,
disc_struct integer NOT NULL,
department integer,
hire_dept_group integer,
discount numeric(4,2),
CONSTRAINT h_disc_detail_pkey PRIMARY KEY (nsuk),
  CONSTRAINT h_disc_detail_fk FOREIGN KEY (disc_struct)
      REFERENCES disc_struct (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT h_disc_detail_fk1 FOREIGN KEY (department)
      REFERENCES hire_dept (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT h_disc_detail_fk2 FOREIGN KEY (hire_dept_group)
      REFERENCES hire_dept_group (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
create table user_language (
language_code char(6),
country text,
CONSTRAINT user_language_pkey PRIMARY KEY (language_code));

create table product_type_lang (
id serial NOT NULL,
product_type_id integer NOT NULL,
user_language char(6) NOT NULL,
description text
,
CONSTRAINT pt_lang_pkey PRIMARY KEY (id),
CONSTRAINT pt_lang_fk1 FOREIGN KEY (product_type_id)
      REFERENCES product_type (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
CONSTRAINT pt_lang_fk2 FOREIGN KEY (user_language)
      REFERENCES user_language (language_code) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
--
-- Product.cod needs a unique constraint
--
ALTER TABLE product ADD CONSTRAINT product_uq UNIQUE (cod);
--
-- saccess needs foreign key to product
ALTER TABLE saccess ADD CONSTRAINT saccess_product_fk FOREIGN KEY (product)
      REFERENCES product (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;

--
-- haccess needs foreign key to pdesc
--

ALTER TABLE haccess ADD CONSTRAINT haccess_pdesc_fk FOREIGN KEY (pdesc)
      REFERENCES pdesc (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Plant Movement Types
--
create table plant_movement_type (
id serial not null,
description text not null,
CONSTRAINT pmove_type_pk PRIMARY KEY (id));
insert into plant_movement_type values (1, 'Addition');
insert into plant_movement_type values (2, 'Hire');
insert into plant_movement_type values (3, 'Adjusted');
insert into plant_movement_type values (4, 'Return');
insert into plant_movement_type values (5, 'Transfer Out');
insert into plant_movement_type values (6, 'Disposal');
insert into plant_movement_type values (7, 'Sold');
insert into plant_movement_type values (8, 'Unsold');
insert into plant_movement_type values (9, 'Deleted');
insert into plant_movement_type values (10, 'Contract Deleted');
insert into plant_movement_type values (11, 'Invoice');
insert into plant_movement_type values (12, 'Credit Note');
insert into plant_movement_type values (13, 'Transfer In');
insert into plant_movement_type values (14, 'In repair');
insert into plant_movement_type values (15, 'Off Repair');
insert into plant_movement_type values (16, 'Delete Issue');
insert into plant_movement_type values (17, 'Delete Return');
insert into plant_movement_type values (18, 'Off Hire');
ALTER TABLE pmoves ADD CONSTRAINT pmoves_pmove_type_fk FOREIGN KEY (typ)
      REFERENCES plant_movement_type (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
update pmoves set to_stat = null where to_stat = 0;
update pmoves set from_stat = null where from_stat = 0;
ALTER TABLE pmoves ADD CONSTRAINT pmoves_plantstat_fk FOREIGN KEY (from_stat)
      REFERENCES plant_status (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE pmoves ADD CONSTRAINT pmoves_plantstat_fk2 FOREIGN KEY (to_stat)
      REFERENCES plant_status (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Foreign keys on quotations
--
ALTER TABLE q_rental ADD CONSTRAINT q_rental_to_quote_fk FOREIGN KEY (quotation) 
      REFERENCES "quote" (nsuk) MATCH SIMPLE 
      ON UPDATE NO ACTION ON DELETE NO ACTION;

ALTER TABLE q_rental ADD CONSTRAINT q_rental_pdesc_fk FOREIGN KEY (asset_reg, pdesc) 
      REFERENCES pdesc (asset_reg, cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE q_rental ADD CONSTRAINT q_rental_chargeline_fk FOREIGN KEY (charge_line) 
      REFERENCES charge_lines (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE q_rental ADD CONSTRAINT q_rental_to_vat_fk FOREIGN KEY (vcode) 
      REFERENCES vat (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE q_rental ADD CONSTRAINT q_rental_to_operator_fk FOREIGN KEY ("operator") 
      REFERENCES "operator" (cod) MATCH SIMPLE 
      ON UPDATE NO ACTION ON DELETE NO ACTION;

ALTER TABLE q_sale ADD CONSTRAINT q_sale_to_quote_fk FOREIGN KEY (quotation) 
      REFERENCES "quote" (nsuk) MATCH SIMPLE 
      ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE q_sale ADD CONSTRAINT q_sale_to_pt_fk FOREIGN KEY (product_type) 
      REFERENCES product_type (nsuk) MATCH SIMPLE 
      ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE q_sale ADD CONSTRAINT q_sale_to_vat_fk FOREIGN KEY (vcode) 
      REFERENCES vat (cod) MATCH SIMPLE 
      ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE q_sale ADD CONSTRAINT q_sale_to_location_fk FOREIGN KEY ("operator") 
      REFERENCES "operator" (cod) MATCH SIMPLE 
      ON UPDATE NO ACTION ON DELETE NO ACTION;

ALTER TABLE q_disposal ADD CONSTRAINT q_disposal_to_quote FOREIGN KEY (quotation) 
      REFERENCES quote (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE q_disposal ADD CONSTRAINT q_disposal_to_pdesc FOREIGN KEY (asset_reg, pdesc) 
      REFERENCES pdesc (asset_reg, cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE q_disposal ADD CONSTRAINT q_disposal_to_vat_fk FOREIGN KEY (vcode) 
      REFERENCES vat (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE q_disposal ADD CONSTRAINT q_disposal_to_operator_fk FOREIGN KEY ("operator") 
      REFERENCES "operator" (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Foreign key codetail to chead
--

alter table codetail ADD CONSTRAINT codetail_chead_fk FOREIGN KEY ("location", contract)
      REFERENCES chead ("location", contract) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;



 -- Deletes for hmdetail and hmsales 
 -- ref integrity

 --delete from hmdetail
 --where (contract_location,contract) not in (select "location",contract from chead);

 --delete
 --from hmsales
 --where (location,contract) not in (select location, contract from chead);


--delete from ihdetail
--where plant_desc not in ( select cod from pdesc ); 

 --
 -- Foreign key hmdetail to chead
 --    




alter table hmdetail add CONSTRAINT hmdet_chead_fkey FOREIGN KEY (contract_location, contract)
      REFERENCES chead ("location", contract) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Foreign key hmsales to chead
--

alter table hmsales add CONSTRAINT hmsales_chead_fkey FOREIGN KEY (contract_location, contract)
      REFERENCES chead ("location", contract) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
      --
      -- Create unique constraint on pdesc.cod
      --


alter table pdesc add constraint pdesc_cod_uq  UNIQUE (cod);
ALTER TABLE ihdetail ADD CONSTRAINT ihdetail_to_pdesc_fk FOREIGN KEY ("plant_desc") 
      REFERENCES "pdesc" (cod) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Fix pkey on ihead to nsuk and make old key unique
--
alter table ihdetail drop constraint ihdetail_fk;
alter table iodetail drop constraint iodetail_fk;
alter table iddetail drop constraint iddetail_fk;
alter table ihead DROP CONSTRAINT ihead_pkey;
alter table ihead add constraint ihead_pkey PRIMARY KEY (nsuk);
alter table ihead add constraint ihead_uk UNIQUE (doc_type, location, ref, cust);


-- here

select * from ihdetail
where (doc_type,"location",ref,cust)
not in ( select doc_type,"location",ref,cust from ihead );

alter table ihdetail add CONSTRAINT ihdetail_fk FOREIGN KEY (doc_type, "location", ref, cust)
      REFERENCES ihead (doc_type, "location", ref, cust) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;

alter table iodetail add CONSTRAINT iodetail_fk FOREIGN KEY (doc_type, "location", ref, cust)
      REFERENCES ihead (doc_type, "location", ref, cust) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;

alter table iddetail add CONSTRAINT iddetail_fk FOREIGN KEY (doc_type, "location", ref, cust)
      REFERENCES ihead (doc_type, "location", ref, cust) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;

 --
 -- Add foreign key for customer contact to quote
 -- 
 update quote set customer_contact = null where customer_contact = 0;
 alter table quote add CONSTRAINT contact_fk FOREIGN KEY (customer_contact)
      REFERENCES contact (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;

      --
      -- Add Foreign Key for Contact to cheadext
      --

 update cheadext set customer_contact = null where customer_contact = 0;
 alter table cheadext add CONSTRAINT contact_fk FOREIGN KEY (customer_contact)
      REFERENCES contact (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;  
--
-- Add iPad to android_device table
--

insert into android_device_type values (3,'Apple iPad',110.00,0.00,-25.00,0.00); 
insert into android_device values (3,'APPLEIPAD','Timmy DeMutt',1,3,1);  

--
-- Add config line for mobile counter
--

insert into sigcap_config values (2,'dcinthewest@gmail.com', 'JPoint Mobile Hire', 'The following attachment is confirmation of your recent transaction','This transaction undertaken by: ','of Acme Plant Hire', 'xxxxx',0);
--
-- Create contact_photo table
--

CREATE SEQUENCE "contactPhoto_id_seq"
   START 1;
ALTER TABLE "contactPhoto_id_seq" OWNER TO postgres;

CREATE TABLE contact_photo
(
  id integer NOT NULL DEFAULT nextval('"contactPhoto_id_seq"'::regclass),
  photo bytea NOT NULL,
  CONSTRAINT "contactPhotoPK" PRIMARY KEY (id),
  CONSTRAINT photo_contact_fk FOREIGN KEY (id)
      REFERENCES contact (nsuk) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
ALTER TABLE contact_photo OWNER TO postgres;