-- Table: audit_sledger

-- DROP TABLE audit_sledger;

CREATE TABLE trig_sledger
(
  audit_nsuk serial NOT NULL,
  audit_timestamp timestamp without time zone,
  audit_type text,
  ser integer,
  depot integer,
  cod character(7),
  dat date,
  period date,
  typ integer,
  amount numeric(16,2),
  vat numeric(16,2),
  "location" integer,
  contract character(6),
  ref character(10),
  os numeric(16,2),
  source character(2),
  batch integer,
  description character(15),
  txt integer,
  lodgement integer,
  "operator" integer,
  invtype character(1),
  tim time without time zone,
  cash_nominal character(6),
  cash_invoice integer
);
ALTER TABLE audit_sledger OWNER TO postgres;
GRANT ALL ON TABLE audit_sledger TO postgres;
GRANT ALL ON TABLE audit_sledger TO public;


CREATE or REPLACE FUNCTION audit_sledger_trigger() RETURNS trigger AS $body$
    BEGIN
          --
        -- Create a row in emp_audit to reflect the operation performed on emp,
        -- make use of the special variable TG_OP to work out the operation.
        --

	if (TG_OP = 'DELETE') THEN

		IF (OLD.typ != 11) THEN
			return OLD;
		END IF;
	
		insert into trig_sledger values ( default,
		now(),
		TG_OP,
		OLD.ser,
		OLD.depot,
		OLD.cod,
		OLD.dat,
		OLD.period,
		OLD.typ,
		OLD.amount,
		OLD.vat,
		OLD.location,
		OLD.contract,
		OLD.ref,
		OLD.os,
		OLD.source,
		OLD.batch,
		OLD.description,
		OLD.txt,
		OLD.lodgement,
		OLD.operator,
		OLD.invtype,
		OLD.tim,
		OLD.cash_nominal,
		OLD.cash_invoice
		);
		return OLD;
	end if;


        IF (NEW.typ != 11) THEN
		return NEW;
        END IF;
        
	insert into trig_sledger values ( default,
	now(),
	TG_OP,
	NEW.ser,
	NEW.depot,
	NEW.cod,
	NEW.dat,
	NEW.period,
	NEW.typ,
	NEW.amount,
	NEW.vat,
	NEW.location,
	NEW.contract,
	NEW.ref,
	NEW.os,
	NEW.source,
	NEW.batch,
	NEW.description,
	NEW.txt,
	NEW.lodgement,
	NEW.operator,
	NEW.invtype,
	NEW.tim,
	NEW.cash_nominal,
	NEW.cash_invoice
	);
	return NEW;
	
    END;
$body$ LANGUAGE plpgsql;

CREATE TRIGGER sledger_deposit_trigger
AFTER INSERT OR UPDATE OR DELETE ON sledger
    FOR EACH ROW EXECUTE PROCEDURE audit_sledger_trigger();




CREATE or REPLACE FUNCTION block_deposit_trigger() RETURNS trigger AS $body$
    BEGIN
          --
        -- Create a row in emp_audit to reflect the operation performed on emp,
        -- make use of the special variable TG_OP to work out the operation.
        --
        IF (NEW.typ != 11) THEN
		return NEW;
        END IF;

	if (TG_OP = 'UPDATE') THEN

	    if ( OLD.os != NEW.os ) then
		if (OLD.os = 0.00 ) then
	            raise exception 'Attempting to change a deposits zero value, please contact JPoint Support !!';
		else 
		    return NEW;	
		end if;
		return NEW;
	    end if ;
	    return NEW;
	else
		return NEW;
	     
	end if;

	return NEW;
    END;
$body$ LANGUAGE plpgsql;


CREATE TRIGGER block_deposit_trigger
AFTER UPDATE ON sledger
    FOR EACH ROW EXECUTE PROCEDURE block_deposit_trigger();




CREATE or REPLACE FUNCTION block_chdetail_trigger() RETURNS trigger AS $body$
declare chead_dat date;
    BEGIN
          --
        -- Create a row in emp_audit to reflect the operation performed on emp,
        -- make use of the special variable TG_OP to work out the operation.
        --
	if (TG_OP = 'UPDATE') THEN
            if ( OLD.date_from != NEW.date_from ) then

		-- find out the date on the chead

		select dat 
		into chead_dat
		from chead
		where "location" = OLD.location
		and   contract = OLD.contract
		and   cust = OLD.cust;

		if ( not FOUND ) then 
		    return NEW;
		end if;
            
		if ( NEW.date_from < chead_dat ) then
		        raise exception 'Attempting to change a chdetails date to before its cheads date. Please contact JPOINT Support immediately !!';
		else
			return NEW;	
		end if;
	    else 
		return NEW;
	    end if;
	else
	    return NEW;
	end if;

	return NEW;
    END;
$body$ LANGUAGE plpgsql;

CREATE TRIGGER block_chdetail_trigger
AFTER UPDATE ON chdetail
    FOR EACH ROW EXECUTE PROCEDURE block_chdetail_trigger();