-- Function: pl_open_bal(date)

-- DROP FUNCTION pl_open_bal(date);

CREATE OR REPLACE FUNCTION pl_open_bal(pperiod date)
  RETURNS jp_open_bal AS
$BODY$
DECLARE
  Pcreditors      nominal.cod%type;
  Pmovement     decimal(16,2);
  Pmsg          char(60);
  row_found     boolean; 
  result        jp_open_bal;
  base_row      record;    
  w_rows        integer;
BEGIN
 
---
--- 1.  Get the opening balance from the checkpoint previous to the period
---     selected

    result.balance := 0.00;
    
    select account_code
      into Pcreditors
      from reserved_ac
     where reserved_ac.key_name = 'creditor_total';
    
    get diagnostics w_rows = ROW_COUNT;
    
    if w_rows = 0 then
         result.ret_string := 'Creditors Control Account not defined on reserved accounts';
         result.ret_stat := 1;
         return result;
    end if;

    row_found := false;

    for base_row in select sum(0 - amount) as amount, period  
                      from nltrans,nominal
                     where nltrans.source = 'CP'
                       and nltrans.period <= Pperiod
                       and nltrans.cod = nominal.cod
                       and nominal.parent = Pcreditors
                  group by  period
                  order by  period desc
    loop

        row_found = true;
        exit;

    end loop; 
    
    if not row_found then
       result.ret_stat := 1;
       result.ret_string := 'There is no nominal checkpoint before the requested period';
       return result;
    end if;

    ---
    --- We now have the balance on the debtors nominal and the period
    ---

    Pmovement := 0.00;

   
    select sum(amount)
    into Pmovement
    from pledger
    where period >= base_row.period
      and period < pperiod
      and supplier is not null;
   

    if Pmovement is null then
        Pmovement := 0.00;
    end if;
    
    result.ret_stat := 1;
    result.ret_string = 'Success';
    result.balance := base_row.amount + Pmovement; 
    return result;

END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION pl_open_bal(date) OWNER TO postgres;
