CREATE OR REPLACE FUNCTION ex( query text )
  RETURNS VOID AS
$BODY$

declare err	sp_status;

begin


	BEGIN
		execute query;

	EXCEPTION
		WHEN others THEN
	             --do nothing we want to run all the way through;
	END;


end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

create or replace function bankrec()
returns VOID AS
$BODY$

begin

perform ex('

create table banknlxref
  (
    nsuk serial not null ,
    nltrans_id integer,
    bank_rec_id integer,
    cleared integer,
    primary key (nsuk)
  );

');

perform ex('

create table bank_rec
  (
    nsuk serial not null ,
    statement_date date,
    opening_balance decimal(16,2),
    closing_balance decimal(16,2),
    operator_id smallint,
    date_processed date,
    primary key (nsuk)
  );

');


perform ex('
alter table bank_rec add constraint fk_oper_id foreign key (operator_id) references operator ;
');

perform ex('
alter table banknlxref add constraint fk_bankrecid foreign key (bank_rec_id) references bank_rec ;
');

perform ex('
create index banknlxref_nltrans_id on banknlxref(nltrans_id);
');

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION bankrec() OWNER TO postgres;

select * from bankrec();

create or replace function misc1()
returns VOID AS
$BODY$
begin

perform ex('

CREATE TABLE nominal_lookup
(
  nsuk serial NOT NULL,
  external_code character(20),
  nominal_code character(6),
  primary key (nsuk)
);

');

perform ex('create unique index ctnomlookup1 on nominal_lookup(external_code);');

perform ex('create unique index ctnomlookup2 on nominal_lookup(nominal_code);');

perform ex('alter table vat add primary key(cod);');

perform ex('
CREATE TABLE vat_lookup
(
  nsuk serial NOT NULL,
  vcode smallint,
  external_code integer,
  primary key (nsuk)
);

');

perform ex('create unique index ctvatlookup1 on vat_lookup(external_code);');
perform ex('create unique index ctvatlookup2 on vat_lookup(vcode);');

perform ex('alter table vat_lookup add constraint fk_vatlookup1 foreign key (vcode) references vat ');

perform ex('
CREATE TABLE non_disc_item
(
    nsuk serial NOT NULL,
    pdesc_asset_reg character(1),
    pdesc_id character(4),
    product_id integer,
    product_type_id integer,
    primary key (nsuk)
);
');

perform ex('alter table non_disc_item add constraint non_desc_fk1 foreign key (product_id) references product; ');
perform ex('alter table non_disc_item add constraint non_desc_fk2 foreign key (product_type_id) references product_type;');
perform ex('alter table non_disc_item add constraint non_desc_fk3 foreign key (pdesc_id) references pdesc (cod);');




end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc1() OWNER TO postgres;

select * from misc1();

create or replace function misc20100510()
returns VOID AS
$BODY$

begin

--perform ex(' sql query here');

--perform ex(' sql query 2  in here ');

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc20100510() OWNER TO postgres;

-- Function: verify_dr_nl(date, integer)

-- DROP FUNCTION verify_dr_nl(date, integer);

CREATE OR REPLACE FUNCTION verify_dr_nl(theperiod date, psuppress_ok integer)
  RETURNS SETOF jp_verify AS
$BODY$
declare Pslsum          pledger.amount%type;
        Pnlsum          pledger.amount%type;
        Pdebtor         reserved_ac.account_code%type;
        Pcount          integer;
        Pnlperiod       nltrans.period%type;
        Pdirection      char(20);
        debtors_row     record;
        nominal_row     record;
        ret_row         jp_verify;
        w_cnt           integer;
        y_cnt           integer;
        count_row       record;
        this_period     sledger.period%type;

begin

    this_period := theperiod;

    select account_code
      into Pdebtor
      from reserved_ac
     where key_name = 'debtors';

    for debtors_row in
      select a.source, a.batch, sum(a.amount) as total
       from sledger a
      where a.period = this_period
        and a.cod is not null
      group by 1,2

    loop

        ret_row.direction := 'Sales to Nominal';

        select count(*)
          into Pcount
         from nltrans
        where source = debtors_row.source
          and batch = debtors_row.batch;

        if Pcount = 0 then
            ret_row.comment := '[REBUILD] Batch does not exist';
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := null;
            ret_row.nominal_period := null;
            return next ret_row;
            continue;
        end if;

        select sum(amount)
         into Pnlsum
         from nltrans
        where source = debtors_row.source
          and batch = debtors_row.batch;

        if Pnlsum is null then
            Pnlsum := 0.00;
        end if;

        if Pnlsum <> 0.00 then
            ret_row.comment := '[REBUILD] Batch is not in balance';
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := Pnlsum;
            ret_row.nominal_period := null;
            return next ret_row;
        end if;

        select sum(amount)
         into Pnlsum
         from nltrans
        where source = debtors_row.source
          and batch = debtors_row.batch
          and cod = Pdebtor;

         if Pnlsum is null then
            ret_row.comment := '[REBUILD] No Debtor Posting';
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := Pnlsum;
            ret_row.nominal_period := null;
            return next ret_row;
        end if;


        if Pnlsum is null then
            Pnlsum := 0.00;
        end if;

        if Pnlsum <> debtors_row.total then
            ret_row.comment := 'Totals do not agree';
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := Pnlsum;
            ret_row.nominal_period := null;
            return next ret_row;
        end if;

        w_cnt := 0;

        for count_row in
            select  distinct period
             from nltrans
             where source = debtors_row.source
              and batch = debtors_row.batch
        loop
            w_cnt := w_cnt + 1;
        end loop;

        y_cnt := 0;

        for count_row in
            select  distinct period
             from sledger
             where source = debtors_row.source
              and batch = debtors_row.batch
        loop
            y_cnt := y_cnt + 1;
        end loop;

        if w_cnt > 1 or y_cnt > 1 then
            if w_cnt > 1 then
                ret_row.comment := '[REBUILD] Batch posted to ' || w_cnt || ' periods';
            else
                ret_row.comment := '[SPLIT] Batch posted to ' || y_cnt || ' periods';
            end if;
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := Pnlsum;
            ret_row.nominal_period := Pnlperiod;
            return next ret_row;
        else

            select distinct period
              into Pnlperiod
             from nltrans
            where source = debtors_row.source
              and batch = debtors_row.batch;

            if Pnlperiod <> ThePeriod then
                ret_row.comment := '[MOVE] Batch posted to a different period';
                ret_row.source := debtors_row.source;
                ret_row.batch := debtors_row.batch;
                ret_row.subledger_total := debtors_row.total;
                ret_row.nominal_total := Pnlsum;
                ret_row.nominal_period := Pnlperiod;
                return next ret_row;
            end if;
        end if;

        if Psuppress_ok <> 1 then
            ret_row.comment := 'Batch OK';
            ret_row.source := debtors_row.source;
            ret_row.batch := debtors_row.batch;
            ret_row.subledger_total := debtors_row.total;
            ret_row.nominal_total := Pnlsum;
            ret_row.nominal_period := Pnlperiod;
            return next ret_row;
        end if;

    end loop;

    Pslsum := 0.00;

    for nominal_row in
     select source, batch, sum(amount) as total
       from nltrans
      where nltrans.period = this_period
        and nltrans.cod = Pdebtor
        and source <> 'CP'
      group by 1,2
      UNION
      select nltrans.source, batch, sum(amount) as total
       from nltrans , nlbatch
      where nltrans.source = nlbatch.source
        and nltrans.batch = nlbatch.seq
        and nltrans.period is null
        and nltrans.cod = Pdebtor
        and nltrans.source <> 'CP'
        and nlbatch.period = this_period
      group by 1,2

    loop

      ret_row.direction := 'Nominal To Sales';

	-- dont check sledger if the nominal debtor is zero
	if nominal_row.total <> 0 THEN

		select count(*)
		  into Pcount
		 from sledger
		where source = nominal_row.source
		  and batch = nominal_row.batch;

		Pslsum := 0.00;

		if Pcount = 0 THEN
		    ret_row.comment := '[NO FIX] Batch does not exist';
		    ret_row.source := nominal_row.source;
		    ret_row.batch := nominal_row.batch;
		    ret_row.subledger_total := 0.00;
		    ret_row.nominal_total := nominal_row.total;
		    ret_row.nominal_period := null;
		    return next ret_row;
		    continue;
		end if;



		select sum(amount)
		  into Pslsum
		  from sledger
		 where source = nominal_row.source
		   and batch = nominal_row.batch;

		 if nominal_row.total <> Pslsum then
		    ret_row.comment := '[REBUILD] Totals do not agree';
		    ret_row.source := nominal_row.source;
		    ret_row.batch := nominal_row.batch;
		    ret_row.subledger_total := 0.00;
		    ret_row.nominal_total := nominal_row.total;
		    ret_row.nominal_period := null;
		    return next ret_row;
		end if;

	end if;



        w_cnt := 0;

        for count_row in
            select distinct period
              from sledger
             where source = nominal_row.source
               and batch = nominal_row.batch
        loop
            w_cnt := w_cnt + 1;
        end loop;

        if w_cnt > 1 then
            ret_row.comment := '[SPLIT] Batch posted to ' || w_cnt || ' periods';
            ret_row.source := nominal_row.source;
            ret_row.batch := nominal_row.batch;
            ret_row.subledger_total := Pslsum;
            ret_row.nominal_total := nominal_row.total;
            ret_row.nominal_period := Pnlperiod;
            return next ret_row;
        else

            select distinct period
              into Pnlperiod                -- Use Pnlperiod to store the slperiod in this case
              from sledger
             where source = nominal_row.source
              and batch = nominal_row.batch  ;

            if Pnlperiod <> ThePeriod then
                ret_row.comment := '[MOVE] Batch posted to a different period';
                ret_row.source := nominal_row.source;
                ret_row.batch := nominal_row.batch;
                ret_row.subledger_total := Pslsum;
                ret_row.nominal_total := nominal_row.total;
                ret_row.nominal_period := Pnlperiod;
                return next ret_row;
            end if;
        end if;

        if Psuppress_ok <> 1 then
            ret_row.comment := 'Batch OK';
            ret_row.source := nominal_row.source;
            ret_row.batch := nominal_row.batch;
            ret_row.subledger_total := Pslsum;
            ret_row.nominal_total := nominal_row.total;
            ret_row.nominal_period := Pnlperiod;
            return next ret_row;
        end if;

    end loop;

   return;

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION verify_dr_nl(date, integer) OWNER TO postgres;


create or replace function misc20100601()
returns VOID AS
$BODY$

begin

--perform ex(' sql query here');

--perform ex(' sql query 2  in here ');

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc20100601() OWNER TO postgres;

-- Function: sledger_at(integer, character, date, integer)

-- DROP FUNCTION sledger_at(integer, character, date, integer);

CREATE OR REPLACE FUNCTION sledger_at(p_depot integer, p_cod character, p_date date, p_reverse integer)
  RETURNS SETOF sledger AS
$BODY$
declare sledger_row           sledger%rowtype;
        Palloc                sledger.os%type;
        Pledger_period        sledger.period%type;
        allocationNumber      integer;
	Psallocee	      sallocee%rowtype;
	Psallocer	      sallocer%rowtype;
begin

  select * from jp_period(p_date) into Pledger_period;
  Palloc := 0;

  for sledger_row in
     select *
     from sledger sl
     where sl.depot = p_depot and sl.cod = p_cod
     and sl.dat <= p_date
     order by sl.dat
  loop

     if p_reverse = 1 then
         if sledger_row.typ < 10 then
             select sum(ee.amount + ee.discount)
                 into Palloc
             from sallocee ee, sallocper per
             where ee.ee = sledger_row.ser
             and ee.allocation = per.allocation
             and per.dat > p_date;
         else
             select sum(er.amount)
                 into Palloc
             from sallocer er, sallocper per
             where er.er = sledger_row.ser
             and er.allocation = per.allocation
             and per.dat > p_date;

         end if;

         -- If the alloc amount is null set it to zero
         IF Palloc is null then
             Palloc := 0;
         end if;
     end if;

     if sledger_row.typ < 10 then
         sledger_row.os = sledger_row.os + Palloc;
     else
         sledger_row.os = sledger_row.os + Palloc;
     end if;

     -- return row if its in same period with a non zero os
     if sledger_row.os <> 0 or sledger_row.period = Pledger_period then

         return next sledger_row;

     else

	     -- return row if its involved in an allocation in the same period either allocee or allocer. Even though its os is ZERO

	     -- If its a payee

	     if sledger_row.typ < 10 then

		-- get the sallocee row

		select *
		into Psallocee
		from sallocee
		left join sallocper on ( sallocee.allocation = sallocper.allocation )
		where sallocee.ee = sledger_row.ser
		and   sallocper.period = Pledger_period;

		if NOT FOUND then
		    continue; -- if we have no sallocee
		else
		     return next sledger_row;
		end if;

	     else
		-- If its a payer

		-- get the sallocer row

		select *
		into Psallocer
		from sallocer
		left join sallocper on ( sallocer.allocation = sallocper.allocation )
		where sallocer.er = sledger_row.ser
		and   sallocper.period = Pledger_period;

		if NOT FOUND then
		    continue; -- if we have no sallocee
		else
		    return next sledger_row;
		end if;


	     end if;
	end if;
  end loop;

  return;

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION sledger_at(integer, character, date, integer) OWNER TO postgres;

create or replace function misc20100824()
returns VOID AS
$BODY$

begin

--perform ex('');

--perform ex(' sql query 2  in here ');


perform ex('
create table op_permissions(
id serial,
operator_id int ,
opt 	int ,
description text,
auth int,
primary key (id)
);
');

perform ex('alter table op_permissions add constraint fk_operator foreign key (operator_id) references operator;');




end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION misc20100824() OWNER TO postgres;

select * from misc20100824();


-- Function: verify_io_sum(date, date)

-- DROP FUNCTION verify_io_sum(date, date);

CREATE OR REPLACE FUNCTION verify_io_sum(pfrom date, pto date)
  RETURNS sp_status AS
$BODY$

declare err             sp_status;

        Pcod            char(6);

        Pdesc           char(30);

        Pvalue          decimal(16,2);



begin



BEGIN

    drop table t_verify_io_sum;

EXCEPTION

    when undefined_table THEN

END;



 create temp table t_verify_io_sum AS

 select a.nominal as code, d.description, sum(a.goods * c.mult) as value

    from iodetail a, ihead b, signs c, nominal d

    where b.period between Pfrom and Pto

      AND A.DOC_TYPE = B.DOC_TYpe

      AND A.location = b.location

      and a.ref = b.ref

      and a.cust = b.cust

      and a.doc_type = c.doc_type

      and a.nominal = d.cod

        group by 1,2;





    err.ret_stat := 0;

    err.ret_string := 'Success';

    return err;



end;

$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION verify_io_sum(date, date) OWNER TO postgres;

